import { type Extension } from "@codemirror/state";
import { Awareness, LoroDoc, UndoManager } from "loro-crdt";
import { type UserState } from "./awareness.ts";
export { undo, redo } from "./undo.ts";
export { getTextFromDoc } from "./sync.ts";
/**
 * It is used to sync the document with the remote users.
 *
 * @param doc - LoroDoc instance
 * @returns Extension
 */
export declare const LoroSyncPlugin: (doc: LoroDoc) => Extension;
/**
 * LoroAwarenessPlugin is a plugin that adds awareness to the editor.
 * It is used to sync the cursor position and selection of the editor with the remote users.
 *
 * @param doc - LoroDoc instance
 * @param awareness - Awareness instance
 * @param user - User info
 * @param getUserId - Function to get the user id. If not provided, the doc's peerId will be used.
 * @returns Extension[]
 */
export declare const LoroAwarenessPlugin: (doc: LoroDoc, awareness: Awareness, user: UserState, getUserId?: () => string) => Extension[];
/**
 * LoroUndoPlugin is a plugin that adds undo/redo to the editor.
 *
 * @param doc - LoroDoc instance
 * @param undoManager - UndoManager instance
 * @returns Extension[]
 */
export declare const LoroUndoPlugin: (doc: LoroDoc, undoManager: UndoManager) => Extension[];
export declare function LoroExtensions(doc: LoroDoc, awareness?: {
    user: UserState;
    awareness: Awareness;
    getUserId?: () => string;
}, undoManager?: UndoManager): Extension;
